/***************************************************************************
                          kallery.cpp  -  description
                             -------------------
    begin                : Tue Oct 16 19:03:20 EEST 2001
    copyright            : (C) 2001 by Mantia Andras
    email                : amantia@freemail.hu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 /*Own include files*/
#include "kallery.h"

 /*System includes*/
#include <stdio.h>
#include <time.h>
#include <sys/types.h>

/*Utility includes*/
#include <magick/api.h>

/*KDE header files*/
#include <kcolordialog.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kio/job.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kcmdlineargs.h>
#include <kcharsets.h>

/*QT header files*/
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmultilineedit.h>
#include <qpixmap.h>
#include <qprogressdialog.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtextstream.h>
#include <qtoolbutton.h>


Kallery::Kallery(QWidget *parent, const char *name) : KWizard(parent, name, FALSE)
{
  kdDebug() << "Kallery is starting up." << "\n";

  appBaseDir = KGlobal::dirs()->findResourceDir("data","kallery/images/kallery.png")+"kallery/";
  kdDebug() << "Kallery data dir is: " << appBaseDir << "\n";

  if (appBaseDir == "kallery/")
  {
    KMessageBox::information(this,i18n("The base data directory [$KDEDIR/share/apps/kallery] seems to be invalid, so the generation may fail.\n")+
                            i18n("You may need to do a \"make install\".\n"),i18n("Warning"));
  }

  // set text of buttons again, so our translation will be used
  // available by extract and merge messages
  nextButton()->setText(i18n("&Next"));

  //kFunction contains the non GUI releated tasks (except the progress bar)
  kFunction = new KalleryFunctions(this,appBaseDir);

  KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
  kFunction->printToStdOut = args->isSet("verbose");
  setCaption(i18n("Kallery wizard"));

  addFirstPage();
  addSelectSourcesPage();
  addThumbSourcePage();
  addImageThumbPage();
  addTargetPathPage();
  addDescSourcePage();
  addImageOptionsPage();
  addHtmlOptionsPage();
  addImageHtmlPage();
  addLastPage();

  kdDebug() << "All pages were added\n";

  connect(this,SIGNAL(selected(const QString &)),this,SLOT(slotPageHasChanged( const QString &)));
  connect(this,SIGNAL(helpClicked()),this,SLOT(slotShowHelp()));

  setDefaultOptions();
  initFilters();
  loadTemplates();
}

Kallery::~Kallery()
{
  delete kFunction;
}

/** No descriptions */
void Kallery::slotOpenProject()
{
  QString projectFileName;

  projectFileName = KFileDialog::getOpenFileName("","*.kallery\n*|All files",this,i18n("Open project"));
  if (!projectFileName.isEmpty())
  {
    if (kFunction->openProject(projectFileName,options) )
    {
      baseDir = QFileInfo(options.galleryFile).dirPath()+"/";
      if (options.generatedProject)
      {
        projectFileName = " - "+QFileInfo(options.projectFileName).baseName()+" ["+i18n("Generated")+"]";
      }
      else
      {
        projectFileName = " - "+QFileInfo(options.projectFileName).baseName()+" ["+i18n("Original")+"]";
      }
      setCaption(i18n("Kallery wizard")+projectFileName);
    }
    else
    {
      setDefaultOptions();
    }
  }
}

/** No descriptions */
void Kallery::addFirstPage()
{
  firstPage = new KFirstPage;
  firstPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));

  setNextEnabled(firstPage,TRUE);
  setHelpEnabled(firstPage,TRUE);
  addPage(firstPage,i18n("STEP 1: Getting started..."));
  connect(firstPage->ChooseProjectBtn,SIGNAL(clicked()),this,SLOT(slotOpenProject()));
  connect(firstPage->OpenProjectBtn,SIGNAL(clicked()),this,SLOT(slotOpenProject()));
}

/** Save the options set on the current page, make a check and if everything is ok,
go to the next page. */
void Kallery::next()
{
  bool goToNextPage;

  goToNextPage = TRUE;

  if (currentPage() == firstPage) goToNextPage = processFirstPage();
  if (currentPage() == selectSourcesPage) goToNextPage = processSelectSourcesPage();
  if (currentPage() == thumbSourcePage) goToNextPage = processThumbSourcePage();
  if (currentPage() == descSourcePage) goToNextPage = processDescSourcePage();
  if (currentPage() == targetPathPage) goToNextPage = processTargetPathPage();
  if (currentPage() == htmlOptionsPage) goToNextPage = processHtmlOptionsPage();
  if (currentPage() == imageOptionsPage) goToNextPage = processImageOptionsPage();
  if (currentPage() == imageHtmlPage) goToNextPage = processImageHtmlPage();

  if (goToNextPage) KWizard::next();
}

/** No descriptions */
void Kallery::addSelectSourcesPage()
{
  selectSourcesPage = new KSelectSourcesPage;

  selectSourcesPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(selectSourcesPage,i18n("STEP 2: Select the sources..."));
  setNextEnabled(selectSourcesPage,TRUE);

  connect(selectSourcesPage->AddBtn,SIGNAL(clicked()),this,SLOT(slotLoadSourceFiles()));
  connect(selectSourcesPage->DeleteBtn,SIGNAL(clicked()),this,SLOT(slotRemoveSourceFile()));

  connect(selectSourcesPage->ImageUpBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageSourceList()));
  connect(selectSourcesPage->ImageDownBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageSourceList()));
  connect(selectSourcesPage->ImageHomeBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageSourceList()));
  connect(selectSourcesPage->ImageEndBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageSourceList()));
}

/** No descriptions */
void Kallery::slotLoadSourceFiles()
{
  options.imageList +=  KFileDialog::getOpenFileNames("",imageFilter,this,i18n("Select image files"));

  selectSourcesPage->ImageListBox->clear();
  selectSourcesPage->ImageListBox->insertStringList(options.imageList,-1);
  selectSourcesPage->ImageListBox->setSelected(0,TRUE);
  selectSourcesPage->ImageNumTxt->setText(i18n("%1 items selected.").arg(options.imageList.count()));

  for (uint i = options.descList.count(); i < options.imageList.count(); i ++)
  {
    options.descList.append("");
  }
}

/** No descriptions */
void Kallery::addTargetPathPage()
{
  targetPathPage = new KTargetPathPage;

  targetPathPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(targetPathPage,i18n("STEP 5: Where to put the result files?"));
  setNextEnabled(targetPathPage,TRUE);

  connect(targetPathPage->ThumbDirSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectThumbDir()));
  connect(targetPathPage->ImageDirSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectImageDir()));
  connect(targetPathPage->GallerySelBtn,SIGNAL(clicked()),this,SLOT(slotSelectGalleryFile()));
}

/** No descriptions */
void Kallery::slotSelectThumbDir()
{
  kFunction->selectDir(i18n("Select target thumbnail directory"),targetPathPage->ThumbDirText,options.thumbDestDir);
}

/** No descriptions */
void Kallery::slotSelectImageDir()
{
  kFunction->selectDir(i18n("Select target image directory"),targetPathPage->ImageDirText,options.imageDestDir);
}

/** No descriptions */
void Kallery::slotSelectGalleryFile()
{
  QString tempStr;

  tempStr = KFileDialog::getOpenFileName(0,"*",this,i18n("Select gallery file directory and name"));
  if(!tempStr.isEmpty())
  {
    options.galleryFile = tempStr;
  }
  targetPathPage->GalleryText->setText(options.galleryFile);
}

/** No descriptions */
void Kallery::slotLoadThumbnails()
{
  if (thumbSourcePage->LoadThumbBtn->isChecked())
  {
    options.thumbList +=  KFileDialog::getOpenFileNames(0,imageFilter,this,i18n("Select thumbnail files"));

    thumbSourcePage->ImageListBox->clear();
    thumbSourcePage->ImageListBox->insertStringList(options.thumbList,-1);
    thumbSourcePage->ImageListBox->setSelected(0,TRUE);
    thumbSourcePage->ImageNumTxt->setText( i18n(" %1 items selected.").arg(options.thumbList.count()));
  }
}

/* Add the image-thumbnail matching page*/
void Kallery::addImageThumbPage()
{
  imageThumbPage = new KImageThumbPage;

  imageThumbPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(imageThumbPage,i18n("STEP 4: Match image and thumbnail files...")) ;
  setNextEnabled(targetPathPage,TRUE);

  connect(imageThumbPage->ImageUpBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageList()));
  connect(imageThumbPage->ImageDownBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageList()));
  connect(imageThumbPage->ImageHomeBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageList()));
  connect(imageThumbPage->ImageEndBtn,SIGNAL(pressed()),this,SLOT(slotModifyImageList()));
  connect(imageThumbPage->ImageListBox,SIGNAL(highlighted(int)),this,SLOT(slotSelectInBothList(int)));

  connect(imageThumbPage->ThumbUpBtn,SIGNAL(pressed()),this,SLOT(slotModifyThumbList()));
  connect(imageThumbPage->ThumbDownBtn,SIGNAL(pressed()),this,SLOT(slotModifyThumbList()));
  connect(imageThumbPage->ThumbHomeBtn,SIGNAL(pressed()),this,SLOT(slotModifyThumbList()));
  connect(imageThumbPage->ThumbEndBtn,SIGNAL(pressed()),this,SLOT(slotModifyThumbList()));
  connect(imageThumbPage->ThumbListBox,SIGNAL(highlighted(int)),this,SLOT(slotSelectInBothList(int)));
}

/* Add the last wizard page*/
void Kallery::addLastPage()
{
  lastPage = new KLastPage;

  lastPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(lastPage,i18n("STEP 10: Final step, revise the options.")) ;
  setBackEnabled(lastPage,TRUE);
  setFinishEnabled(lastPage,TRUE);
}

/** Move up or down the items in the image list, selectSources page */
void Kallery::slotModifyImageSourceList()
{
  if (selectSourcesPage->ImageHomeBtn->isDown())
  {
    kFunction->moveInListAndBox(selectSourcesPage->ImageListBox,options.imageList,&options.descList,0,-1);
    selectSourcesPage->ImageHomeBtn->setDown(FALSE);
  }
  if (selectSourcesPage->ImageUpBtn->isDown())
  {
    kFunction->moveInListAndBox(selectSourcesPage->ImageListBox,options.imageList,&options.descList,-1,0);
    selectSourcesPage->ImageUpBtn->setDown(FALSE);
  }
  if (selectSourcesPage->ImageDownBtn->isDown())
  {
    kFunction->moveInListAndBox(selectSourcesPage->ImageListBox,options.imageList,&options.descList,1,0);
    selectSourcesPage->ImageDownBtn->setDown(FALSE);
  }
  if (selectSourcesPage->ImageEndBtn->isDown())
  {
    kFunction->moveInListAndBox(selectSourcesPage->ImageListBox,options.imageList,&options.descList,0,1);
    selectSourcesPage->ImageEndBtn->setDown(FALSE);
  }
}

/** Move up or down the items in the image list */                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  //Del by KDevelop:/** No descriptions */
void Kallery::slotModifyImageList()
{
  if (imageThumbPage->ImageHomeBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ImageListBox,options.imageList,&options.descList,0,-1);
    imageThumbPage->ImageHomeBtn->setDown(FALSE);
  }
  if (imageThumbPage->ImageUpBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ImageListBox,options.imageList,&options.descList,-1,0);
    imageThumbPage->ImageUpBtn->setDown(FALSE);
  }
  if (imageThumbPage->ImageDownBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ImageListBox,options.imageList,&options.descList,1,0);
    imageThumbPage->ImageDownBtn->setDown(FALSE);
  }
  if (imageThumbPage->ImageEndBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ImageListBox,options.imageList,&options.descList,0,1);
    imageThumbPage->ImageEndBtn->setDown(FALSE);
  }
}

/** Move up or down the items in the thumbnail list */                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  //Del by KDevelop:/** No descriptions */
void Kallery::slotModifyThumbList()
{
  if (imageThumbPage->ThumbHomeBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ThumbListBox,options.thumbList,0,0,-1);
    imageThumbPage->ThumbHomeBtn->setDown(FALSE);
  }
  if (imageThumbPage->ThumbUpBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ThumbListBox,options.thumbList,0,-1,0);
    imageThumbPage->ThumbUpBtn->setDown(FALSE);
  }
  if (imageThumbPage->ThumbDownBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ThumbListBox,options.thumbList,0,1,0);
    imageThumbPage->ThumbDownBtn->setDown(FALSE);
  }
  if (imageThumbPage->ThumbEndBtn->isDown())
  {
    kFunction->moveInListAndBox(imageThumbPage->ThumbListBox,options.thumbList,0,0,1);
    imageThumbPage->ThumbEndBtn->setDown(FALSE);
  }
}

/** Synchronize the selections in the image and thumbnail lists */
void Kallery::slotSelectInBothList(int index)
{
  imageThumbPage->ImageListBox->setCurrentItem(index);
  imageThumbPage->ThumbListBox->setCurrentItem(index);
}

/** Build the thumbnail setting page*/
void Kallery::addThumbSourcePage()
{
  thumbSourcePage = new KThumbSourcePage;
  thumbSourcePage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(thumbSourcePage,i18n("STEP 3: Thumbnails..."));
  setNextEnabled(thumbSourcePage,TRUE);

 connect(thumbSourcePage->AddBtn,SIGNAL(clicked()),this,SLOT(slotLoadThumbnails()));
 connect(thumbSourcePage->DeleteBtn,SIGNAL(clicked()),this,SLOT(slotRemoveThumbnail()));
}


/** Build the descriptions setting page*/
void Kallery::addDescSourcePage()
{
  descSourcePage = new KDescSourcePage;
  descSourcePage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(descSourcePage,i18n("STEP 6: Describe the images..."));
  setNextEnabled(descSourcePage,TRUE);

  connect(descSourcePage->DescSelBtn,SIGNAL(clicked()),this,SLOT(slotLoadDescriptions()));
  connect(descSourcePage->LoadDescBtn,SIGNAL(clicked()),this,SLOT(slotLoadDescriptions()));
  connect(descSourcePage->NextDescBtn,SIGNAL(clicked()),this,SLOT(slotNextDescription()));
  connect(descSourcePage->PrevDescBtn,SIGNAL(clicked()),this,SLOT(slotPreviousDescription()));
}

/** Delete an image from the image list */
void Kallery::slotRemoveSourceFile()
{
  kFunction->deleteFromList(selectSourcesPage->ImageListBox,options.imageList,selectSourcesPage->ImageNumTxt, &options.descList);
}

/** Delete a thumbnail item from the thumbnail list*/
void Kallery::slotRemoveThumbnail()
{
  kFunction->deleteFromList(thumbSourcePage->ImageListBox,options.thumbList,thumbSourcePage->ImageNumTxt, 0L);
}


/** Setup the default values for the next page, which will be displayed. */
void Kallery::slotPageHasChanged(const QString &title)
{
  kdDebug() << "Current page will be: " << title << "\n";

  if (currentPage() == selectSourcesPage)  setSelectSourcesPage();
  if (currentPage() == thumbSourcePage) setThumbSourcePage();
  if (currentPage() == descSourcePage) setDescSourcePage();
  if (currentPage() == imageThumbPage) setImageThumbPage();
  if (currentPage() == targetPathPage) setTargetPathPage();
  if (currentPage() == imageOptionsPage) setImageOptionsPage();
  if (currentPage() == htmlOptionsPage) setHtmlOptionsPage();
  if (currentPage() == imageHtmlPage) setImageHtmlPage();
  if (currentPage() == lastPage) setLastPage();
}

/** No descriptions */
void Kallery::addHtmlOptionsPage()
{
  htmlOptionsPage = new KHtmlOptionsPage;

  htmlOptionsPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(htmlOptionsPage,i18n("STEP 8: Gallery look & feel..."));
  setNextEnabled(htmlOptionsPage,TRUE);

  connect(htmlOptionsPage->TableColorBtn,SIGNAL(clicked()),this,SLOT(slotSelectTableColor()));
  connect(htmlOptionsPage->BackColorBtn,SIGNAL(clicked()),this,SLOT(slotSelectBackColor()));
  connect(htmlOptionsPage->TextColorBtn,SIGNAL(clicked()),this,SLOT(slotSelectTextColor()));
  connect(htmlOptionsPage->GalleryCSSSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectGalleryCSS()));
  connect(htmlOptionsPage->BeforeHtmlSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectBeforeHtml()));
  connect(htmlOptionsPage->AfterHtmlSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectAfterHtml()));
}

/** Do what it was written for. */
void Kallery::accept()
{
  QString projectFileName;

  if (options.relativePaths)
  {
    QString CSSDir;
    if (!options.imageCSSFile.isEmpty())
    {
      CSSDir = QFileInfo(options.imageCSSFile).dirPath();
      if (kFunction->makeRelativePath(options.imageDestDir,CSSDir))
      {
        options.imageCSSFile = CSSDir + QFileInfo(options.imageCSSFile).fileName();
      }
    }
    if (!options.galleryCSSFile.isEmpty())
    {
      CSSDir = QFileInfo(options.galleryCSSFile).dirPath();
      if (kFunction->makeRelativePath(baseDir,CSSDir))
      {
        options.galleryCSSFile = CSSDir + QFileInfo(options.galleryCSSFile).fileName();
      }
    }
    kFunction->makeRelativePath(baseDir,options.thumbDestDir);
    kFunction->makeRelativePath(baseDir,options.imageDestDir);
    if (options.thumbSetting == LoadThumbnails)
    {
      QString thumbDir;
      for (uint i = 0; i < options.thumbList.count(); i++)
      {
        thumbDir = QFileInfo(options.thumbList[i]).dirPath();
        kFunction->makeRelativePath(baseDir,thumbDir);
        options.thumbList[i] = thumbDir + QFileInfo(options.thumbList[i]).fileName();
      }
    }
  }

  if (options.thumbSetting == GenerateThumbnails)
  {
    options.thumbList.clear();
    for (uint i = 0; i < options.imageList.count(); i++)
    {
      options.thumbList.append(options.thumbDestDir+QFileInfo(options.imageList[i]).baseName()+"."+options.thumbFormat);
    }
  }

  if (KMessageBox::questionYesNo(this,i18n("Do you want to save the project?"), i18n("Save project")) == KMessageBox::Yes)
  {
    projectFileName = KFileDialog::getSaveFileName(QFileInfo(options.fileName).dirPath(),"*.kallery\n*|All files",this,i18n("Save project"));
    if (!projectFileName.isEmpty())
    {
      if (QFileInfo(projectFileName).extension().isEmpty())
      {
        projectFileName += ".kallery";
      }
      //if we save the generated project under a new name, it will became an original one.
      if ((options.generatedProject) && (options.fileName != projectFileName))
      {
        options.generatedProject = FALSE;
      }
      if (!kFunction->saveProject(projectFileName,options))
      {
//      DestroyMagick();
        return;
      }
    }
  }

  progress=new QProgressDialog(i18n("Converting..."), i18n("Cancel"),
                                 2*options.imageList.count(),this,"prg",TRUE);
  progress->show();
  progress->setCaption(i18n("Converting..."));
  progress->setProgress(1);
  kapp->processEvents();
  if (!kFunction->convertImages(options,progress))
  {
    delete progress;
    kapp->deref();
    DestroyMagick();
    return;
  }

  progress->setCaption(i18n("Creating web pages..."));
  progress->setLabelText("");
  kFunction->createHtmls(options,progress);

  delete progress;

  if (options.generatedProject)
  {
    if (KMessageBox::questionYesNo(this,i18n("Shall I synchronize the original project with the generated? (Recommended.)"), i18n("Save project")) == KMessageBox::Yes)
    {
      OptionType tempOptions;
      kFunction->openProject(options.projectFileName,tempOptions);
// Set all but the source image list, and image/thumbnail conversion options to the actual settings also in the main project file
      options.generatedProject = FALSE;
      options.imageList = tempOptions.imageList;
      options.thumbSetting = tempOptions.thumbSetting;
      options.thumbFormat = tempOptions.thumbFormat;
      options.thumbWidth = tempOptions.thumbWidth;
      options.thumbHeight = tempOptions.thumbHeight;
      options.convertImages =  tempOptions.convertImages;
      options.fullFormat = tempOptions.fullFormat;
      options.quality = tempOptions.quality;
      options.resize = tempOptions.resize;
      options.keepRatio = tempOptions.keepRatio;
      options.onlyLarger = tempOptions.onlyLarger;
      options.fullWidth = tempOptions.fullWidth;
      options.fullHeight = tempOptions.fullHeight;
      options.insertCopyright = tempOptions.insertCopyright;
      options.copyright = tempOptions.copyright;
      kFunction->saveProject(options.projectFileName,options);
    }
  }
  else
  {
    if (KMessageBox::questionYesNo(this,i18n("Shall I save a generated project file? (Recommended.)"), i18n("Save project")) == KMessageBox::Yes)
    {
      projectFileName = QFileInfo(projectFileName).dirPath()+"/"+QFileInfo(projectFileName).baseName()+"_generated.kallery";
      if (options.thumbSetting == GenerateThumbnails)
      {
        options.thumbSetting = LoadThumbnails;
      }
      options.convertImages = CopyOnly;
      options.generatedProject = TRUE;
      kFunction->saveProject(projectFileName,options);
    }
  }

  KMessageBox::information(this,i18n("The image gallery was generated. Enjoy it!"),i18n("Game over"));

  kapp->deref();
  DestroyMagick();
//  KWizard::accept();
}

void Kallery::reject()
{
  kapp->deref();
  KWizard::reject();
}

/** No descriptions */
void Kallery::slotSelectTableColor()
{
  QColor tmpColor = options.tableColor;

  KColorDialog::getColor(tmpColor,htmlOptionsPage);
  htmlOptionsPage->TableColorFrame->setBackgroundColor(tmpColor);
}

/** No descriptions */
void Kallery::slotSelectBackColor()
{
  QColor tmpColor = options.backColor;

  KColorDialog::getColor(tmpColor,htmlOptionsPage);
  htmlOptionsPage->BackColorFrame->setBackgroundColor(tmpColor);
}

/** No descriptions */
void Kallery::slotSelectTextColor()
{
  QColor tmpColor = options.textColor;

  KColorDialog::getColor(tmpColor,htmlOptionsPage);
  htmlOptionsPage->TextColorFrame->setBackgroundColor(tmpColor);
}

/** No descriptions */
void Kallery::slotSelectImageBackColor()
{
  QColor tmpColor = options.imageBackColor;

  KColorDialog::getColor(tmpColor,imageHtmlPage);
  imageHtmlPage->BackColorFrame->setBackgroundColor(tmpColor);
}

/** No descriptions */
void Kallery::slotSelectImageTextColor()
{
  QColor tmpColor = options.imageTextColor;
  KColorDialog::getColor(tmpColor,imageHtmlPage);
  imageHtmlPage->TextColorFrame->setBackgroundColor(tmpColor);
}

/** No descriptions */
void Kallery::slotSelectNextPic()
{
  QString tempStr;
  tempStr = KFileDialog::getOpenFileName(0,imageFilter,this,i18n("Select picture for Next button"));
  if (! tempStr.isEmpty()) {options.nextPic = tempStr;}
  if (!QFileInfo(options.nextPic).exists()) {options.nextPic = "";}
  imageHtmlPage->NextPic->setPixmap(QPixmap(options.nextPic));
}

/** No descriptions */
void Kallery::slotSelectPrevPic()
{
  QString tempStr;
  tempStr = KFileDialog::getOpenFileName(0,imageFilter,this,i18n("Select picture for Previous button"));
  if (! tempStr.isEmpty())
  {
    options.prevPic = tempStr;
  }
  if (!QFileInfo(options.prevPic).exists())
  {
    options.prevPic = "";
  }
  imageHtmlPage->PrevPic->setPixmap(QPixmap(options.prevPic));
}

/** No descriptions */
void Kallery::slotSelectBackPic()
{
  QString tempStr;
  tempStr = KFileDialog::getOpenFileName(0,imageFilter,this,i18n("Select picture for Back button"));
  if (! tempStr.isEmpty())
  {
    options.backPic = tempStr;
  }
  if (!QFileInfo(options.backPic).exists())
  {
    options.backPic = "";
  }
  imageHtmlPage->BackPic->setPixmap(QPixmap(options.backPic));
}

/** No descriptions */
void Kallery::addImageOptionsPage()
{
  imageOptionsPage = new KImageOptionsPage;

  imageOptionsPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(imageOptionsPage,i18n("STEP 7: Image options..."));
  setNextEnabled(imageOptionsPage,TRUE);

  connect(imageOptionsPage->QualitySlider,SIGNAL(valueChanged(int)),this,SLOT(slotQualityValueChanged(int)));
}

#ifndef USE_SYSTEM_CP
/** No descriptions */
void Kallery::slotResult(KIO::Job * job)
{
  if (job->error())
   job->showErrorDialog();
  kapp->deref();
}
#endif

/** No descriptions */
void Kallery::addImageHtmlPage()
{
  imageHtmlPage = new KImageHtmlPage;

  imageHtmlPage->LogoPixmap->setPixmap(QPixmap(appBaseDir+"images/kallery.png",0,QPixmap::Auto));
  addPage(imageHtmlPage,i18n("STEP 9: Fullsize image look & feel."));
  setNextEnabled(imageHtmlPage,TRUE);

  connect(imageHtmlPage->OpenNextPic,SIGNAL(clicked()),this,SLOT(slotSelectNextPic()));
  connect(imageHtmlPage->OpenPrevPic,SIGNAL(clicked()),this,SLOT(slotSelectPrevPic()));
  connect(imageHtmlPage->OpenBackPic,SIGNAL(clicked()),this,SLOT(slotSelectBackPic()));
  connect(imageHtmlPage->BackColorBtn,SIGNAL(clicked()),this,SLOT(slotSelectImageBackColor()));
  connect(imageHtmlPage->TextColorBtn,SIGNAL(clicked()),this,SLOT(slotSelectImageTextColor()));
  connect(imageHtmlPage->ImageCSSSelBtn,SIGNAL(clicked()),this,SLOT(slotSelectImageCSS()));
}

/** No descriptions */
void Kallery::slotLoadDescriptions()
{
  QString tempStr;

  tempStr = KFileDialog::getOpenFileName(QFileInfo(options.fileName).dirPath(),"*",this,i18n("Select a description file"));
  if(!tempStr.isEmpty())
  {
    kFunction->readDescriptions(tempStr,options.descList);
    for (uint i = options.descList.count(); i < options.imageList.count(); i ++)
    {
      options.descList.append("");
    }
    options.descSetting = LoadDescriptions;
    setDescSourcePage();
  }
}

/** No descriptions */
void Kallery::slotNextDescription()
{
  QStringList::Iterator it;

  descSourcePage->PrevDescBtn->setEnabled(TRUE);
  if (DescPosition +1 > options.descList.count())
  {
    options.descList.append(descSourcePage->DescEdit->text());
  }
  else
  {
    it = options.descList.at(DescPosition);
    it = options.descList.remove(it);
    options.descList.insert(it,descSourcePage->DescEdit->text());
  }

  DescPosition ++;
  if (DescPosition+1 == options.imageList.count())
  {
    descSourcePage->NextDescBtn->setEnabled(FALSE);
  }

  if (DescPosition < options.descList.count())
  {
    descSourcePage->DescEdit->setText(options.descList[DescPosition]);
  }
  else
  {
    descSourcePage->DescEdit->clear();
  }
  descSourcePage->ImageNameTxt->setText(i18n("Describe the image: %1").arg(QFileInfo(options.imageList[DescPosition]).fileName()));
  pm.load(options.imageList[DescPosition]);
  dx = double(descSourcePage->descPixmap->width()) / double(pm.width());
  matrix.reset();
  matrix.scale(dx, dx);
  pm_scale = pm.xForm(matrix);
//  descSourcePage->DescPic->setPaletteBackgroundPixmap(pm_scale);
  descSourcePage->descPixmap->setPixmap(pm_scale);
}

/** No descriptions */
void Kallery::slotPreviousDescription()
{
  QStringList::Iterator it;

  descSourcePage->NextDescBtn->setEnabled(TRUE);
  if (DescPosition +1 > options.descList.count())
  {
    options.descList.append(descSourcePage->DescEdit->text());
  }
  else
  {
    it = options.descList.at(DescPosition);
    it = options.descList.remove(it);
    options.descList.insert(it,descSourcePage->DescEdit->text());
  }
  DescPosition --;
  if (DescPosition == 0)
  {
    descSourcePage->PrevDescBtn->setEnabled(FALSE);
  }
  descSourcePage->DescEdit->setText(options.descList[DescPosition]);
  descSourcePage->ImageNameTxt->setText(i18n("Describe the image: %1").arg(QFileInfo(options.imageList[DescPosition]).fileName()));
  pm.load(options.imageList[DescPosition]);
  dx = double(descSourcePage->descPixmap->width()) / double(pm.width());
  matrix.reset();
  matrix.scale(dx, dx);
  pm_scale = pm.xForm(matrix);
//  descSourcePage->DescPic->setPaletteBackgroundPixmap(pm_scale);
  descSourcePage->descPixmap->setPixmap(pm_scale);
}

/** No descriptions */
void Kallery::setDefaultOptions()
{
  /* set default options */
  options.galleryTitle = i18n("Image gallery");
  options.galleryFile = QDir::currentDirPath()+"/myGallery/gallery.htm";
  options.imageDestDir = QDir::currentDirPath()+"/myGallery/fullsize";
  options.thumbDestDir = QDir::currentDirPath()+"/myGallery/thumbs";
  options.relativePaths = true;
  options.imageList.clear();
  options.thumbList.clear();
  options.descList.clear();
  options.thumbSetting = GenerateThumbnails;
  options.descSetting = UseFilename;
  options.thumbWidth = 100;
  options.thumbHeight = 75;
  options.fullWidth = 600;
  options.fullHeight = 450;
  options.convertImages = Convert;
  options.resize = true;
  options.keepRatio = true;
  options.onlyLarger = true;
  options.thumbFormat = "jpg";
  options.fullFormat = "jpg";
  options.quality = 80; // == 80%
  options.columnNum = 3;
  options.useHtmlImages = true;
  options.textButtons = true;
  options.nextStr = i18n("Next");
  options.prevStr = i18n("Previous");
  options.backStr = i18n("Back");
  options.nextPic = appBaseDir+"images/next.png";
  options.prevPic = appBaseDir+"images/prev.png";
  options.backPic = appBaseDir+"images/back.png";
  options.imageBackColor.setNamedColor("#000000");
  options.imageTextColor.setNamedColor("#FFFFFF");
  options.insertCopyright = false;
  options.copyright = " " + i18n("This year, You");
  options.galleryTemplate = i18n("Default template");
  options.fullsizeTemplate = i18n("Buttons above the image");
  options.language = i18n("United Kingdom (gb)");
  options.encoding = "utf8";
  options.useFrames = false;
  options.backColor.setNamedColor("#000000");
  options.tableColor.setNamedColor("#000000");
  options.textColor.setNamedColor("#FFFFFF");
  options.galleryCSSFile = "";
  options.beforeHtmlFile = "";
  options.afterHtmlFile = "";
  options.imageCSSFile = "";
  options.projectFileName = "";
  options.generatedProject = false;
  options.fileName = "";

  setCaption(i18n("Kallery wizard"));
  baseDir = QFileInfo(options.galleryFile).dirPath()+"/";
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processFirstPage()
{
  if (firstPage->StartNewBtn->isChecked())
    setDefaultOptions();
  // make a backup from orginal options for some checks while generating gallery
  options_old = options;
  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processSelectSourcesPage()
{
  if (options.imageList.isEmpty())
  {
    KMessageBox::error(this,i18n("There is no image file selected.\nPlease select at least one image file."),i18n("Empty list error"),TRUE);
    return FALSE;
  }
  //Check for duplicate filename, they cause problems. It can be a long process...
  QString tmpString = "";
  uint k = 0;
  for (uint i = 0; i < options.imageList.count();  i++)
  {
    for (uint j = i + 1; j < options.imageList.count(); j++)
    {
      if (QFileInfo(options.imageList[i]).fileName() == QFileInfo(options.imageList[j]).fileName())
      {
        k++;
        if (k < 11 )
          tmpString += options.imageList[i] + "  <->  " + options.imageList[j] + "\n";
      }
    }
  }

  if (k > 10)
    tmpString += i18n("\n... and %1 other files are also conflicting.").arg(k-10);
  if (tmpString.isEmpty())
  {
    return TRUE;
  }
  else
  {
    KMessageBox::error(this,i18n("Conflicting/duplicate filenames were found: \n\n %1").arg(tmpString),i18n("Duplicate filename error"),TRUE);
    return FALSE;
  }
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processThumbSourcePage()
{
  if (thumbSourcePage->NoThumbBtn->isChecked()) options.thumbSetting = NoThumbnails;
  if (thumbSourcePage->GenerateThumbBtn->isChecked()) options.thumbSetting = GenerateThumbnails;
  if (thumbSourcePage->LoadThumbBtn->isChecked()) options.thumbSetting = LoadThumbnails;

  if ( (options.thumbSetting == LoadThumbnails) && (options.thumbList.count() != options.imageList.count()) )
  {
    KMessageBox::error(this,i18n("The number of thumbnail files is not equal with the number of image files!\n\n %1 thumbnails, but\n %2 image files were selected.") \
                                                  .arg(options.thumbList.count()).arg(options.imageList.count()),i18n("Number mismatch error"),TRUE);
    return FALSE;
  }

  targetPathPage->ThumbDirLabel->setEnabled(options.thumbSetting == GenerateThumbnails);
  targetPathPage->ThumbDirText->setEnabled(options.thumbSetting == GenerateThumbnails);
  targetPathPage->ThumbDirSelBtn->setEnabled(options.thumbSetting == GenerateThumbnails);

  /*Skip the thumbnail and image matching page if there were no thumbnails selected */
  setAppropriate(imageThumbPage,options.thumbSetting == LoadThumbnails );

  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processDescSourcePage()
{
  slotNextDescription();
  if (descSourcePage->NoDescBtn->isChecked()) options.descSetting = NoDescriptions;
  if (descSourcePage->FilenameDescBtn->isChecked()) options.descSetting = UseFilename;
  if (descSourcePage->LoadDescBtn->isChecked()) options.descSetting = LoadDescriptions;
  if (descSourcePage->EnterDescBtn->isChecked()) options.descSetting = EnterDescriptions;

  /*If we enter or load the descriptions, do a check.*/
  if ((options.descSetting > UseFilename) && (options.descList.count() != options.imageList.count()) )
  {
    KMessageBox::error(this,i18n("The number of descriptions is not equal with the number of image files!\n\n %1 descriptions, but\n %2 image files were selected.") \
                                                  .arg(options.descList.count()).arg(options.imageList.count()),i18n("Number mismatch error"),TRUE);
    return FALSE;
  }

  if (options.descSetting == UseFilename )
  {
    options.descList.clear();
    for (uint i = 0; i < options.imageList.count(); i++)
      options.descList.append(QFileInfo(options.imageList[i]).baseName());
  }
  if (options.descSetting == NoDescriptions )
  {
    options.descList.clear();
    for (uint i = 0; i < options.imageList.count(); i++)
      options.descList.append("");
  }

  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processTargetPathPage()
{
  /*Read the directory and file strings from the edit boxes*/
  options.thumbDestDir = QDir::cleanDirPath(targetPathPage->ThumbDirText->text())+"/";
  options.imageDestDir = QDir::cleanDirPath(targetPathPage->ImageDirText->text())+"/";
  options.galleryFile = targetPathPage->GalleryText->text();
  baseDir = QFileInfo(options.galleryFile).dirPath()+"/";

  if ( (options.thumbSetting == GenerateThumbnails) && (options.thumbDestDir == options.imageDestDir) )
  {
    KMessageBox::error(this,i18n("The image and thumbnail destination directory cannot be the same!"),i18n("Error"));
    return FALSE;
  }

  /*Create the missing directories and set the relative paths if needed */

  if ( !kFunction->checkAndCreateDir(baseDir) ||
        !kFunction->checkAndCreateDir(options.imageDestDir) ||
        ( (options.thumbSetting == GenerateThumbnails) && (!kFunction->checkAndCreateDir(options.thumbDestDir)) )||
        (options.galleryFile == "") )
  {
    KMessageBox::error(this,i18n("The directories are invalid, or there is no gallery file specified!\n")+
                             i18n("Check them and try again."),i18n("Error"));
    return FALSE;
  }

  options.relativePaths = targetPathPage->RelativePaths->isChecked();

  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processHtmlOptionsPage()
{
  options.galleryTitle = htmlOptionsPage->PageTitleEdit->text();
  options.language = htmlOptionsPage->GalleryLanguageCombo->currentText();
  int i = htmlOptionsPage->GalleryTemplateCombo->currentItem();
  options.galleryTemplateFile = galleryTemplateList[i].left(galleryTemplateList[i].find('|'));
  options.galleryTemplate =  htmlOptionsPage->GalleryTemplateCombo->currentText();
  options.useFrames =  htmlOptionsPage->UseFrames->isChecked();
  options.columnNum =  htmlOptionsPage->ColumnNumSpinBox->value();
  options.tableColor = htmlOptionsPage->TableColorFrame->backgroundColor();
  options.backColor = htmlOptionsPage->BackColorFrame->backgroundColor();
  options.textColor = htmlOptionsPage->TextColorFrame->backgroundColor();
  options.encoding = htmlOptionsPage->EncodingCombo->currentText();

  if (htmlOptionsPage->UseGalleryCSS->isChecked())
  {
    options.galleryCSSFile = htmlOptionsPage->OwnGalleryCSSName->text();
    if (!QFileInfo(options.galleryCSSFile).exists())
    {
      if (KMessageBox::warningYesNo(this,i18n("The file %1 is missing. Continue anyway?").arg(options.galleryCSSFile),i18n("Missing file")) == KMessageBox::No)
      {
        return FALSE;
      }
    }
  }
  else
  {
    options.galleryCSSFile = "";
  }

  options.beforeHtmlFile =  htmlOptionsPage->BeforeHtmlEdit->text();
  options.afterHtmlFile =  htmlOptionsPage->AfterHtmlEdit->text();

  //Check the existance of the selected files.
  if ((!options.beforeHtmlFile.isEmpty()) && (!QFileInfo(options.beforeHtmlFile).exists()))
  {
    if (KMessageBox::warningYesNo(this,i18n("The file %1 is missing. Continue anyway?").arg(options.beforeHtmlFile),i18n("Missing file")) == KMessageBox::No)
    {
       return FALSE;
    }
  }
  if ((!options.afterHtmlFile.isEmpty()) && (!QFileInfo(options.afterHtmlFile).exists()))
  {
    if (KMessageBox::warningYesNo(this,i18n("The file %1 is missing. Continue anyway?").arg(options.afterHtmlFile),i18n("Missing file")) == KMessageBox::No)
    {
       return FALSE;
    }
  }

  if ((options.galleryTemplateFile.find("gallery_frame.template") == -1) &&
    (options.useFrames))
  {
    if (KMessageBox::warningYesNo(this,i18n("You have selected to use frames, but a non frame template. This is not good.\n Continue anyway?"),i18n("Settings missmatch")) == KMessageBox::No)
    {
      return FALSE;
    }
  }

  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processImageOptionsPage()
{
  if (imageOptionsPage->NoCopyConvertFull->isChecked())
    options.convertImages = NoCopyConvert;
  if (imageOptionsPage->CopyFull->isChecked())
    options.convertImages = CopyOnly;
  if (imageOptionsPage->ConvertFull->isChecked())
    options.convertImages = Convert;
  options.resize = imageOptionsPage->Resize->isChecked();
  options.keepRatio = imageOptionsPage->KeepAspect->isChecked();
  options.onlyLarger = imageOptionsPage->OnlyLarger->isChecked();
  options.fullWidth = imageOptionsPage->FullW->value();
  options.fullHeight = imageOptionsPage->FullH->value();
  options.thumbWidth = imageOptionsPage->ThumbW->value();
  options.thumbHeight = imageOptionsPage->ThumbH->value();
  options.quality = imageOptionsPage->QualitySlider->value();
  options.fullFormat = imageOptionsPage->FullImageFormat->currentText();
  options.fullFormat = options.fullFormat.right(options.fullFormat.length()-options.fullFormat.find("*.",1)-2);
  options.fullFormat.truncate(options.fullFormat.find(")",1));
  options.thumbFormat = imageOptionsPage->ThumbImageFormat->currentText();
  options.thumbFormat = options.thumbFormat.right(options.thumbFormat.length()-options.thumbFormat.find("*.",1)-2);
  options.thumbFormat.truncate(options.thumbFormat.find(")",1));
  options.insertCopyright = imageOptionsPage->InsertCopyright->isChecked();
  options.copyright =  imageOptionsPage->CopyrightTxt->text();

  return TRUE;
}

/** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
go to the next page, otherwise returns FALSE. */
bool Kallery::processImageHtmlPage()
{
  int i = imageHtmlPage->FullsizeTemplateCombo->currentItem();
  options.fullsizeTemplateFile = fullsizeTemplateList[i].left(fullsizeTemplateList[i].find('|'));
  options.fullsizeTemplate = imageHtmlPage->FullsizeTemplateCombo->currentText();
  options.useHtmlImages = imageHtmlPage->UseHtmlPages->isChecked();
  options.textButtons = imageHtmlPage->TextButtonsBtn->isChecked();
  options.nextStr =  imageHtmlPage->NextTextEdit->text();
  options.prevStr =  imageHtmlPage->PrevTextEdit->text();
  options.backStr =  imageHtmlPage->BackTextEdit->text();
  options.imageBackColor = imageHtmlPage->BackColorFrame->backgroundColor();
  options.imageTextColor = imageHtmlPage->TextColorFrame->backgroundColor();

  if (imageHtmlPage->UseImageCSS->isChecked())
  {
    options.imageCSSFile = imageHtmlPage->OwnImageCSSName->text();
    if (!QFileInfo(options.imageCSSFile).exists())
    {
      if (KMessageBox::warningYesNo(this,i18n("The file %1 is missing. Continue anyway?").arg(options.imageCSSFile),i18n("Missing file")) == KMessageBox::No)
      {
        return FALSE;
      }
    }
  }
  else
  {
    options.imageCSSFile = "";
  }

  return TRUE;
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setSelectSourcesPage()
{
  selectSourcesPage->ImageListBox->clear();
  selectSourcesPage->ImageListBox->insertStringList(options.imageList,-1);
  selectSourcesPage->ImageNumTxt->setText(i18n("%1 items selected.").arg(options.imageList.count()));
  selectSourcesPage->ImageListBox->setSelected(0,TRUE);
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setThumbSourcePage()
{
  thumbSourcePage->GenerateThumbBtn->setChecked(options.thumbSetting == GenerateThumbnails);
  thumbSourcePage->LoadThumbBtn->setChecked(options.thumbSetting == LoadThumbnails);
  thumbSourcePage->NoThumbBtn->setChecked(options.thumbSetting == NoThumbnails);
  thumbSourcePage->ImageListBox->clear();
  thumbSourcePage->ImageListBox->insertStringList(options.thumbList,-1);
  thumbSourcePage->ImageNumTxt->setText(i18n("%1 items selected.").arg(options.thumbList.count()));
  thumbSourcePage->ImageListBox->setSelected(0,TRUE);
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setDescSourcePage()
{
  descSourcePage->EnterDescBtn->setChecked(options.descSetting == EnterDescriptions);
  descSourcePage->LoadDescBtn->setChecked(options.descSetting == LoadDescriptions);
  descSourcePage->FilenameDescBtn->setChecked(options.descSetting == UseFilename);
  descSourcePage->NoDescBtn->setChecked(options.descSetting == NoDescriptions);
  descSourcePage->PrevDescBtn->setEnabled(FALSE);
  descSourcePage->NextDescBtn->setEnabled(TRUE);
  descSourcePage-> ImageNameTxt->setText(i18n("Describe the image: %1").arg(QFileInfo(options.imageList[0]).fileName()));
  DescPosition = 0;
  if (options.descList.count() > 1)
  {
    descSourcePage->DescEdit->setText(options.descList[0]);
    pm.load(options.imageList[0]);
    dx = double(descSourcePage->descPixmap->width()) / double(pm.width());
    matrix.reset();
    matrix.scale(dx, dx);
    pm_scale = pm.xForm(matrix);
//    descSourcePage->DescPic->setPaletteBackgroundPixmap(pm_scale);
    descSourcePage->descPixmap->setPixmap(pm_scale);
  }
  else
  {
    descSourcePage->DescEdit->clear();
  }
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setTargetPathPage()
{
  targetPathPage->ImageDirText->setText(options.imageDestDir);
  targetPathPage->ThumbDirText->setText(options.thumbDestDir);
  targetPathPage->GalleryText->setText(options.galleryFile);
  targetPathPage->RelativePaths->setChecked(options.relativePaths);
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setImageOptionsPage()
{
  int pos;
  int num;
  int thumbFormatNo;
  int fullFormatNo;
  QString tmpString;

  tmpString = imageFilter;
  num = tmpString.contains('\n');
  imageOptionsPage->ThumbImageFormat->clear();
  imageOptionsPage->FullImageFormat->clear();
  for (int i=0 ; i < num; i++)
  {
    pos = tmpString.find('\n',1);
    if (i > 1)
    {
      imageOptionsPage->ThumbImageFormat->insertItem(tmpString.left(pos));
      imageOptionsPage->FullImageFormat->insertItem(tmpString.left(pos));
      if (tmpString.left(pos).find(QRegExp("."+options.thumbFormat+"\\)"),0) != -1)
      {
        thumbFormatNo = i - 2;
      }
      if (tmpString.left(pos).find(QRegExp("."+options.fullFormat+"\\)"),0) != -1)
      {
        fullFormatNo = i - 2;
      }
    }
    pos = tmpString.find('|',pos+1);
    tmpString.remove(0,pos+1);
  }

  imageOptionsPage->ThumbImageFormat->setCurrentItem(thumbFormatNo);
  imageOptionsPage->FullImageFormat->setCurrentItem(fullFormatNo);
  imageOptionsPage->QualityTxt->setText(QString("[ %1% ]").arg(options.quality));
  imageOptionsPage->QualitySlider->setValue(options.quality);

  imageOptionsPage->ThumbFrame->setEnabled(options.thumbSetting == GenerateThumbnails);
  switch (options.convertImages)
  {
    case NoCopyConvert:  imageOptionsPage->NoCopyConvertFull->setChecked(TRUE);
                         break;
    case CopyOnly: imageOptionsPage->CopyFull->setChecked(TRUE);
                   break;
    case Convert:  imageOptionsPage->ConvertFull->setChecked(TRUE);
                   break;
  }

  imageOptionsPage->Resize->setChecked(options.resize);
  imageOptionsPage->KeepAspect->setChecked(options.keepRatio);
  imageOptionsPage->OnlyLarger->setChecked(options.onlyLarger);
  imageOptionsPage->ThumbW->setValue(options.thumbWidth);
  imageOptionsPage->ThumbH->setValue(options.thumbHeight);
  imageOptionsPage->FullW->setValue(options.fullWidth);
  imageOptionsPage->FullH->setValue(options.fullHeight);
  imageOptionsPage->InsertCopyright->setChecked(options.insertCopyright);
  imageOptionsPage->CopyrightTxt->setText(options.copyright);
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setHtmlOptionsPage()
{
//FIXME: This should load the list of languages, and not the list of countries!!
  loadCountryList(htmlOptionsPage->GalleryLanguageCombo);
  for (int i = 0; i < htmlOptionsPage->GalleryLanguageCombo->count(); i++)
  {
    htmlOptionsPage->GalleryLanguageCombo->setCurrentItem(i);
    if (htmlOptionsPage->GalleryLanguageCombo->currentText() == options.language)
    {
      break;
    }
  }

  htmlOptionsPage->GalleryTemplateCombo->clear();
  for (uint i = 0; i < galleryTemplateList.count(); i++)
  {
    htmlOptionsPage->GalleryTemplateCombo->insertItem(galleryTemplateList[i].mid(galleryTemplateList[i].find('|')+1));
  }

  for (int i = htmlOptionsPage->GalleryTemplateCombo->count() -1; i >=0; i--)
  {
    htmlOptionsPage->GalleryTemplateCombo->setCurrentItem(i);
    if (htmlOptionsPage->GalleryTemplateCombo->currentText() == options.galleryTemplate)
    {
      break;
    }
  }

  QStringList encodings(KGlobal::charsets()->availableEncodingNames());
  int pos = 0;
  for (uint i=0; i < encodings.count(); i++) 
  {
    htmlOptionsPage->EncodingCombo->insertItem(encodings[i]);
    if (encodings[i] == options.encoding)
      pos = i;
  }
  htmlOptionsPage->EncodingCombo->setCurrentItem(pos);
  htmlOptionsPage->PageTitleEdit->setText(options.galleryTitle);
  htmlOptionsPage->ColumnNumSpinBox->setValue(options.columnNum);
  htmlOptionsPage->UseFrames->setChecked(options.useFrames);
  htmlOptionsPage->TableColorFrame->setBackgroundColor(options.tableColor);
  htmlOptionsPage->BackColorFrame->setBackgroundColor(options.backColor);
  htmlOptionsPage->TextColorFrame->setBackgroundColor(options.textColor);

  if (options.galleryCSSFile.isEmpty())
  {
    htmlOptionsPage->UseGalleryCSS->setChecked(FALSE);
  }
  else
  {
    htmlOptionsPage->UseGalleryCSS->setChecked(TRUE);
    htmlOptionsPage->OwnGalleryCSSName->setText(options.galleryCSSFile);
  }

  htmlOptionsPage->BeforeHtmlEdit->setText(options.beforeHtmlFile);
  htmlOptionsPage->AfterHtmlEdit->setText(options.afterHtmlFile);
}

//Load the list of avaliable countries and put them into a combo box
//This code is based on kdebase/kcontrol/locale/locale.cpp.
void Kallery::loadCountryList(QComboBox *combo)
{
  QString sub = QString::fromLatin1("l10n/");
  // clear the list
  combo->clear();
  // add all languages to the list
  QStringList countrylist = KGlobal::dirs()->findAllResources("locale",
                               sub + QString::fromLatin1("*/entry.desktop"));
  countrylist.sort();

  for(QStringList::ConstIterator it=++countrylist.begin();it!=countrylist.end();++it)
  {
    KSimpleConfig entry(*it);
    entry.setGroup(QString::fromLatin1("KCM Locale"));
    QString name = entry.readEntry(QString::fromLatin1("Name"),
                       KGlobal::locale()->translate("without name"));
    QString tag = *it;
    int index = tag.findRev('/');
    tag.truncate(index);
    index = tag.findRev('/');
    tag = tag.mid(index+1);
    combo->insertItem(name+" ("+tag+")");
  }
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setImageHtmlPage()
{
  imageHtmlPage->FullsizeTemplateCombo->clear();
  for (uint i = 0; i < fullsizeTemplateList.count(); i++)
  {
    imageHtmlPage->FullsizeTemplateCombo->insertItem(fullsizeTemplateList[i].mid(fullsizeTemplateList[i].find('|')+1));
  }

  for (int i =  imageHtmlPage->FullsizeTemplateCombo->count() -1; i >=0; i--)
  {
    imageHtmlPage->FullsizeTemplateCombo->setCurrentItem(i);
    if (imageHtmlPage->FullsizeTemplateCombo->currentText() == options.fullsizeTemplate)
    {
      break;
    }
  }

  imageHtmlPage->UseHtmlPages->setChecked(options.useHtmlImages);
  imageHtmlPage->TextButtonsBtn->setChecked(options.textButtons);
  imageHtmlPage->GraphButtonsBtn->setChecked(!options.textButtons);
  imageHtmlPage->NextTextEdit->setText(options.nextStr);
  imageHtmlPage->PrevTextEdit->setText(options.prevStr);
  imageHtmlPage->BackTextEdit->setText(options.backStr);
  imageHtmlPage->NextPic->setPixmap(QPixmap(options.nextPic));
  imageHtmlPage->PrevPic->setPixmap(QPixmap(options.prevPic));
  imageHtmlPage->BackPic->setPixmap(QPixmap(options.backPic));
  imageHtmlPage->BackColorFrame->setBackgroundColor(options.imageBackColor);
  imageHtmlPage->TextColorFrame->setBackgroundColor(options.imageTextColor);

  if (options.imageCSSFile.isEmpty())
  {
    imageHtmlPage->UseImageCSS->setChecked(FALSE);
  }
  else
  {
    imageHtmlPage->UseImageCSS->setChecked(TRUE);
    imageHtmlPage->OwnImageCSSName->setText(options.imageCSSFile);
  }
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setLastPage()
{
  QString tmpString;

  lastPage->GalleryTitleTxt->setText(i18n("Gallery title: %1").arg(options.galleryTitle));
  lastPage->GalleryTxt->setText(i18n("Gallery file: %1").arg(options.galleryFile));
  lastPage->LanguageTxt->setText(i18n("Country: %1").arg(options.language));

  lastPage->EncodingTxt->setText(i18n("Encoding: %1").arg(options.encoding));

  switch (options.thumbSetting)
  {
    case NoThumbnails:
      tmpString = i18n("No thumbnail generation (ignore the next item).");
      break;
    case GenerateThumbnails:
      tmpString = i18n("Generate thumbnails.  [ %1 / %2 x %3 ]").arg(options.thumbFormat).arg(options.thumbWidth)\
      .arg(options.thumbHeight);
      break;
    case LoadThumbnails:
      tmpString = i18n("Load thumbnails from a list.");
      break;
  }
  lastPage->ThumbOptionsTxt->setText(tmpString);
  lastPage->ThumbTxt->setText(i18n("Thumbnail directory: %1").arg(options.thumbDestDir));
  lastPage->FullsizeTxt->setText(i18n("Fullsize directory: %1").arg(options.imageDestDir));

  if (options.relativePaths)
  {
    tmpString  = i18n("Use relative paths.");
  }
  else
  {
    tmpString = i18n("Use absolute paths. (Not recommended.)");
  }
  lastPage->RelativeTxt->setText(tmpString);
  lastPage->ColumnNumTxt->setText(i18n("Columns: %1").arg(options.columnNum));

  tmpString = i18n("%1 images. ").arg(options.imageList.count());
  if (options.convertImages == Convert)
  {
    if (options.resize)
    {
      tmpString +=i18n("[ %1 / %2 x %3 ]").arg(options.fullFormat).arg(options.fullWidth).arg(options.fullHeight);
    }
    else
    {
      tmpString +=i18n("[ %1 / No resize ]").arg(options.fullFormat);
    }

    if (options.insertCopyright)
    {
      lastPage->CopyrightTxt->setText(i18n("Copyright: %1").arg(options.copyright));
    }
    else
    {
      lastPage->CopyrightTxt->setText(i18n("No copyright info inserted"));
    }
  }

   if (options.convertImages == CopyOnly) tmpString += i18n("[Copy the images]");
   lastPage->ImageNumTxt->setText(tmpString);

   if (options.galleryCSSFile.isEmpty())
   {
     tmpString = i18n("CSS is not used for the main gallery.");
   } else
   {
     tmpString = i18n("Gallery CSS file: %1").arg(options.galleryCSSFile);
   }
   lastPage->GalleryCSSTxt->setText(tmpString);

   if (options.imageCSSFile.isEmpty())
   {
     tmpString = i18n("CSS is not used for the fullsize image htmls.");
   } else
   {
     tmpString = i18n("Image CSS file: %1").arg(options.imageCSSFile);
   }
   lastPage->ImageCSSTxt->setText(tmpString);

   lastPage->TableColorFrame->setBackgroundColor(options.tableColor);
   lastPage->BackColorFrame->setBackgroundColor(options.backColor);
   lastPage->TextColorFrame->setBackgroundColor(options.textColor);

   if (options.useHtmlImages)
   {
     tmpString  = i18n("Generate image html pages.");
   }
   else
   {
     tmpString = i18n("Do not generate html page for each image.");
   }
   lastPage->ImageHtmlTxt->setText(tmpString);

   if (options.textButtons)
   {
     tmpString  = i18n("Generate text navigation buttons.");
   }
   else
   {
     tmpString = i18n("Generate bitmap navigation buttons.");
   }
   lastPage->ButtonsTxt->setText(tmpString);
}

/** No descriptions */
void Kallery::slotShowHelp()
{
  if (currentPage() == firstPage) kapp->invokeHelp("first-page","kallery");
  if (currentPage() == selectSourcesPage) kapp->invokeHelp("selectsources-page","kallery");
  if (currentPage() == thumbSourcePage) kapp->invokeHelp("thumbnails-page","kallery");
  if (currentPage() == imageThumbPage) kapp->invokeHelp("imagethumbnail-page","kallery");
  if (currentPage() == targetPathPage) kapp->invokeHelp("targetpath-page","kallery");
  if (currentPage() == descSourcePage) kapp->invokeHelp("description-page","kallery");
  if (currentPage() == imageOptionsPage) kapp->invokeHelp("imageoptions-page","kallery");
  if (currentPage() == htmlOptionsPage) kapp->invokeHelp("htmloptions-page","kallery");
  if (currentPage() == imageHtmlPage) kapp->invokeHelp("imagehtml-page","kallery");
  if (currentPage() == lastPage) kapp->invokeHelp("last-page","kallery");
}

/** Setup this page, because the user will see it just in a moment. */
void Kallery::setImageThumbPage()
{
  QFileInfo fileInfo;

  imageThumbPage->ImageListBox->clear();
  imageThumbPage->ThumbListBox->clear();

  for (uint i =0; i<options.imageList.count();i++)
  {
    fileInfo.setFile(options.imageList[i]) ;
    imageThumbPage->ImageListBox->insertItem(fileInfo.fileName(),-1);
    fileInfo.setFile(options.thumbList[i]) ;
    imageThumbPage->ThumbListBox->insertItem(fileInfo.fileName(),-1);
  }
}

/** No descriptions */
void Kallery::slotSelectGalleryCSS()
{
  QString tempStr;

  tempStr = KFileDialog::getOpenFileName(0,"*",this,i18n("Select the CSS file"));
  if(!tempStr.isEmpty()) {options.galleryCSSFile = tempStr;}
  htmlOptionsPage->OwnGalleryCSSName->setText(options.galleryCSSFile);
}

/** No descriptions */
void Kallery::slotSelectImageCSS()
{
  QString tempStr;

  tempStr = KFileDialog::getOpenFileName(0,"*",this,i18n("Select the CSS file"));
  if(!tempStr.isEmpty())
  {
    options.imageCSSFile = tempStr;
  }
  imageHtmlPage->OwnImageCSSName->setText(options.imageCSSFile);
}

/** The Quality value has changed. Update the text value also. */
void Kallery::slotQualityValueChanged(int value)
{
  imageOptionsPage->QualityTxt->setText(QString("[%1%]").arg(value));
}

/** No descriptions */
void Kallery::slotSelectBeforeHtml()
{
  QString tmpString;

  tmpString = KFileDialog::getOpenFileName("","*",this,i18n("Select a html formatted text file"));
  if (!tmpString.isEmpty())
  {
    htmlOptionsPage->BeforeHtmlEdit->setText(tmpString);
  }
}

/** No descriptions */
void Kallery::slotSelectAfterHtml()
{
  QString tmpString;

  tmpString = KFileDialog::getOpenFileName("","*",this,i18n("Select a html formatted text file"));
  if (!tmpString.isEmpty())
  {
    htmlOptionsPage->AfterHtmlEdit->setText(tmpString);
  }
}

/** Initialize ImageMagick and load the supported filter list. */
void Kallery::initFilters()
{
  InitializeMagick(QDir::currentDirPath());

  const MagickInfo **mInfoList;
  unsigned long numberOfItems;

#ifdef IMAGEMAGICK_6_0
  mInfoList = GetMagickInfoList("*", &numberOfItems);
#else  
  ExceptionInfo exception;
  GetExceptionInfo(&exception);
  mInfoList = GetMagickInfoList("*", &numberOfItems, &exception);
#endif  

  QString tmpStr;
  QString allFilter;
  for (uint i = 0; i < numberOfItems; i++)
  {
    const MagickInfo *mInfo = mInfoList[i];
    tmpStr = mInfo->name;
    if (!tmpStr.isEmpty())
    {
      allFilter += "*."+  tmpStr.lower()+" ";
      imageFilter +=  "*." + tmpStr.lower() +"|"+ QString(mInfo->description)+
                                  " file (*."+ tmpStr.lower() +")\n";
    }
  };
  imageFilter = allFilter +"|All ImageMagick suported image files\n*|All files\n" + imageFilter;

  imageFilter.replace(QRegExp("/"),"");

  kdDebug() << "ImageMagick initalized, filters read." << "\n";
}

/** Load list of the avaliable filters. */
void Kallery::loadTemplates()
{
  QFile tmpFile;
  QString tmpString;

  galleryTemplateList = KGlobal::dirs()-> findAllResources("data","kallery/templates/gallery/*.template");

  for (uint i = 0; i < galleryTemplateList.count(); i++)
  {
    tmpFile.setName(galleryTemplateList[i]);
    tmpFile.open(IO_ReadOnly);
    tmpFile.readLine(tmpString,200);
    tmpFile.readLine(tmpString,200);
    tmpString.replace(QRegExp("[<>!\n-]"),"");
    galleryTemplateList[i] = galleryTemplateList[i]+"|"+i18n(tmpString);
    tmpFile.close();
  }

  fullsizeTemplateList = KGlobal::dirs()-> findAllResources("data","kallery/templates/fullsize/*.template");
  for (uint i = 0; i < fullsizeTemplateList.count(); i++)
  {
    tmpFile.setName(fullsizeTemplateList[i]);
    tmpFile.open(IO_ReadOnly);
    tmpFile.readLine(tmpString,200);
    tmpFile.readLine(tmpString,200);
    tmpString.replace(QRegExp("[<>!\n-]"),"");
    fullsizeTemplateList[i] =fullsizeTemplateList[i] + "|"+i18n(tmpString);
    tmpFile.close();
  }
}

#include "kallery.moc"
