/***************************************************************************
                          kallery.h  -  description
                             -------------------
    begin                : Tue Oct 16 19:03:20 EEST 2001
    copyright            : (C) 2001 by Mantia Andras
    email                : amantia@freemail.hu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KALLERY_H
#define KALLERY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kwizard.h>
#include <krun.h>

#include <qpixmap.h>
#include <qwmatrix.h>

#include "kallery_def.h"
#include "kalleryfunctions.h"
#include "kfirstpage.h"
#include "kselectsourcespage.h"
#include "kthumbsourcepage.h"
#include "kdescsourcepage.h"
#include "ktargetpath.h"
#include "kimagethumbpage.h"
#include "khtmloptionspage.h"
#include "kimagehtmlpage.h"
#include "kimageoptionspage.h"
#include "klastpage.h"

/*Forward QT and KDE class declaration. Header files are included in the .cpp file*/
class QProgressDialog;

/** Kallery is the base class of the project */
class Kallery : public KWizard
{
  Q_OBJECT 
  public:
    /** construtor */
    Kallery(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~Kallery();
  private: // Private attributes
    /**  */
    KalleryFunctions* kFunction;

    KSelectSourcesPage* selectSourcesPage;
    KFirstPage* firstPage;
    KTargetPathPage* targetPathPage;
    KImageThumbPage* imageThumbPage;
    KThumbSourcePage* thumbSourcePage ;
    KDescSourcePage* descSourcePage ;
    KHtmlOptionsPage* htmlOptionsPage;
    KLastPage* lastPage;
    /** Holds different options for generation. */
    struct OptionType options;
    struct OptionType options_old;
    /** Hold the filter patterns for image files suported by ImageMagick. */
    /**  */
    KImageOptionsPage* imageOptionsPage;
    /**  */
    QString imageFilter;
    /** The base path to the application files. */
    QString appBaseDir;
    /** The base path to the gallery file. */
    QString baseDir;
    /**  */
    KImageHtmlPage *imageHtmlPage;
    /** Currently the DescPosition-th image is described. */
    uint DescPosition;
    /**  */
    QProgressDialog* progress;
    /**  */
    QStringList galleryTemplateList;
    /**  */
    QStringList fullsizeTemplateList;

    QPixmap pm;
    QPixmap pm_scale;
    QWMatrix matrix;
    double dx;

  private: // Private methods
    void loadCountryList(QComboBox *combo);
    /** No descriptions */
    void addFirstPage();
    /** No descriptions */
    void addSelectSourcesPage();
    /** No descriptions */
    void addTargetPathPage();
    /** No descriptions */
    void addImageThumbPage();
    /** No descriptions */
    void addHtmlOptionsPage();
    /** No descriptions */
    void addLastPage();
    /** No descriptions */
    void addThumbSourcePage();
    /** No descriptions */
    void addDescSourcePage();
    /** No descriptions */
    void addImageOptionsPage();
    /** No descriptions */
    void addImageHtmlPage();
    /** No descriptions */
    void setDefaultOptions();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processFirstPage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processSelectSourcesPage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processThumbSourcePage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processDescSourcePage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processTargetPathPage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processHtmlOptionsPage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processImageOptionsPage();
    /** Reads the settings from the page. Returns TRUE if everything's ok, and the user can
  go to the next page, otherwise returns FALSE. */
    bool processImageHtmlPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setSelectSourcesPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setThumbSourcePage();
    /** Setup this page, because the user will see it just in a moment. */
    void setDescSourcePage();
    /** Setup this page, because the user will see it just in a moment. */
    void setTargetPathPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setImageOptionsPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setHtmlOptionsPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setImageHtmlPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setLastPage();
    /** Setup this page, because the user will see it just in a moment. */
    void setImageThumbPage();
    /** Load list of the avaliable filters. */
    void loadTemplates();
    /** Initialize ImageMagick and load the supported filter list. */
    void initFilters();

  protected slots: // Protected slots
    /** No descriptions */
    virtual void next();
    /** Do what it is written for. */
    virtual void accept();
    /** Do what it is written for. */
    virtual void reject();

  private slots: // Private slots
    /** No descriptions */
    void slotLoadSourceFiles();
    /** No descriptions */
    void slotSelectThumbDir();
    /** No descriptions */
    void slotSelectImageDir();
    /** No descriptions */
    void slotLoadDescriptions();
    /** No descriptions */
    void slotNextDescription();
    /** No descriptions */
    void slotPreviousDescription();
    /** No descriptions */
    void slotLoadThumbnails();
    /** No descriptions */
    void slotSelectGalleryFile();
    /** No descriptions */
    void slotModifyImageSourceList();
    /** No descriptions */
    void slotModifyImageList();
    /** No descriptions */
    void slotModifyThumbList();
    /** No descriptions */
    void slotRemoveThumbnail();
    /** No descriptions */
    void slotSelectInBothList(int index);
    /** No descriptions */
    void slotOpenProject();
    /** No descriptions */
    void slotRemoveSourceFile();
    /** No descriptions */
    void slotPageHasChanged(const QString & title);
    /** No descriptions */
    void slotSelectBackColor();
    /** No descriptions */
    void slotSelectTableColor();
    /** No descriptions */
    void slotSelectTextColor();
    /** No descriptions */
    void slotSelectImageBackColor();
    /** No descriptions */
    void slotSelectImageTextColor();
    /** No descriptions */
    void slotSelectBackPic();
    /** No descriptions */
    void slotSelectPrevPic();
    /** No descriptions */
    void slotSelectNextPic();
    /** No descriptions */
    void slotShowHelp();
    /** No descriptions */
    void slotSelectImageCSS();
    /** No descriptions */
    void slotSelectGalleryCSS();
    /** The Quality value has changed. Update the text value also. */
    void slotQualityValueChanged(int value);
    /** No descriptions */
    void slotSelectAfterHtml();
    /** No descriptions */
    void slotSelectBeforeHtml();

  public slots:
    #ifndef USE_SYSTEM_CP
    /** No descriptions */
    void slotResult(KIO::Job *);
    #endif
  
};

#endif
