/***************************************************************************
                          kalleryfunctions.h  -  description
                             -------------------
    begin                : Mon Oct 29 2001
    copyright            : (C) 2001 by Mantia Andras
    email                : amantia@freemail.hu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KALLERYFUNCTIONS_H
#define KALLERYFUNCTIONS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


/**This provides the needed functionality. It's methods are called by the Kallery class.
  *@author Mantia Andras
  */

#include "kallery_def.h"

#include <qobject.h>

/*Forward declaration of QT classes. The include files are specified in the .cpp file */
class QString;
class QStringList;
class QListBox;
class QLineEdit;
class QLabel;
class QToolButton;
class QProgressDialog;

class KalleryFunctions : public QObject
{
  Q_OBJECT
  public:
    KalleryFunctions(QWidget* widget, const QString& baseDir );
    virtual ~KalleryFunctions();
    /** Move an item up or down with delta position in a QStringList and the corresponding listbox.
    if special == -1, the item will be the first item,
    if special == 1, the item will became the last item.*/
    void moveInListAndBox(QListBox *listBox,QStringList &itemList,QStringList *descList,
                             int delta, int special);
    /** No descriptions */
    void deleteFromList(QListBox *listBox,QStringList &itemList,QLabel *label, QStringList *descList);
    /** Returns in the dir parameter the relative path of dir based on baseDir. */
    bool makeRelativePath(const QString &baseDir,QString &dir);
    /** Convert the dir to an absolute path, if it was a relative path based on baseDir. */
    bool makeAbsolutePath(const QString & baseDir, QString &dir);
    /** Common method to display a directory selection dialog. */
    void selectDir(const QString &caption,QLineEdit *dirTextLine,QString &dirName);
    /** Checks for the dirName directory and creates if necessary. */
    bool checkAndCreateDir(const QString& dirName );
    /** Saves the project in a file. */
    bool saveProject(const QString& projectFileName,OptionType &options);
    /** No descriptions */
    bool openProject(const QString& projectFileName, OptionType &options);
    /** No descriptions */
    bool convertImages(OptionType & options, QProgressDialog* progress);
    /** Create the gallery html and the htmls for the images, if needed. */
    void createHtmls(OptionType &options,QProgressDialog *progress);
    /** Read the descriptions from a file into the descList. */
    bool readDescriptions(const QString& fileName,QStringList& descList);
  private: // Private attributes
    /**  */
    QWidget* mainWidget;
    /**  */
    bool cancelPressed;
    /**  */
    QString appBaseDir;
  private: // Private methods
    /** Creates a html file with the image and the navigation buttons. */
    void createHtmlForImage(OptionType &options,uint imageIndex,const QString &htmlName,const QString& language);
    /** Write the descriptions from descList into the file. */
    bool writeDescriptions(const QString& fileName,QStringList& descList);
    /** Insert the content of sourceName file into the destStream text stream (another file). */
    void insertFile(QString& sourceName,QTextStream & destStream);
    QString convertEncodingName(const QString &encoding);
  public slots: // Public slots
    /** Called when the user presses the Cancel button. */
    void slotProgressCancelled();
  public: // Public attributes
    /** Print the created file list to stdout or not. */
    bool printToStdOut;
};

#endif
